#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <assert.h>

#include "array.h"
#include "rs.h"

void inform(char *fmt, ...)
{
    va_list arg;

    va_start(arg, fmt);
    vfprintf(stderr, fmt, arg);
    va_end(arg);
}

void panic(char *fmt, ...)
{
    va_list arg;

    va_start(arg, fmt);
    vfprintf(stderr, fmt, arg);
    va_end(arg);

    exit(0);
}

int main()
{	
	Array<unsigned char> sectors(2880*512);
	Array<unsigned char> newsectors(2880*512);
	int i,j,k;
	FILE *fp,*fps;

	// Initialize Reed-Solomon code
	init_rs();
	fps = fopen("diskette", "rb");
	if(!fps)
		panic("No 'diskette' file!\n");
	if(2880 != fread(sectors, 512, 2880, fps))
		panic("Not enough data!\n");
	fclose(fps);

        j=0; k=0;
	for(j=0;j<2880*512;j++) {
		newsectors[j] = sectors[k];
		k+=4096;
		if(k>=2880*512) {
			k-=2880*512;
			k++;
		}
	}

	int iFileLen = 0;

	fp = fopen("datanew", "wb");	
	i = 0;
	switch(eras_dec_rs(((unsigned char *)newsectors)+i*(NN+1), NULL, 0)) {
	case -1:
		panic("File impaired beyond correction\n");
		break;
	case 0:		
		printf(".");
		iFileLen = *(unsigned int *) (unsigned char *)newsectors;
		fwrite(((unsigned char *)newsectors)+i*(NN+1)+20, 
			1, (iFileLen>KK-20)?KK-20:iFileLen, fp);
		break;
	default:
		printf("E");
		iFileLen = *(unsigned int *) (unsigned char *)newsectors;
		fwrite(((unsigned char *)newsectors)+i*(NN+1)+20, 
			1, (iFileLen>KK-20)?KK-20:iFileLen, fp);
		break;
	}
	i++;
	iFileLen -= (iFileLen>KK-20)?KK-20:iFileLen;

	while(iFileLen) {
		switch(eras_dec_rs(((unsigned char *)newsectors)+i*(NN+1), NULL, 0)) {
		case -1:
			panic("File impaired beyond correction\n");
			break;
		case 0:
			if((i%80) == 0)
			    printf(".");
			fwrite(((unsigned char *)newsectors)+i*(NN+1), 
				1, (iFileLen>KK)?KK:iFileLen, fp);
			break;
		default:
			printf("E");
			fwrite(((unsigned char *)newsectors)+i*(NN+1), 
				1, (iFileLen>KK)?KK:iFileLen, fp);
			break;
		}
		i++;
		iFileLen -= (iFileLen>KK)?KK:iFileLen;
	}	
	fclose(fp);
	rename("datanew", (const char*) (unsigned char *)newsectors+4);
	printf("\nCreated file '%s' (%d bytes)", 
	    (unsigned char *)newsectors+4,
	    *((unsigned int *)(unsigned char *)newsectors));

	return 0;
}
